<?php

namespace FiloBlu\StockaFix\Model\Report;

use DateTime;
use FiloBlu\StockaFix\Helper\MailHelper;
use FiloBlu\StockaFix\Model\Report;
use FiloBlu\StockaFix\Model\ReportInterface;
use FiloBlu\StockaFix\Model\ResourceModel\Report\CollectionFactory;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Exception\MailException;
use Magento\Framework\File\Csv;
use Magento\Framework\Filesystem;
use RuntimeException;
use Zend_Db;

class Processor implements ProcessorInterface
{

    /**
     * @var AdapterInterface
     */
    protected $connection;
    /**
     * @var CollectionFactory
     */
    private $reportCollectionFactory;
    /**
     * @var MailHelper
     */
    private $mailHelper;
    /**
     * @var Csv
     */
    private $csvProcessor;
    /**
     * @var DirectoryList
     */
    private $directoryList;
    /**
     * @var Filesystem
     */
    private $filesystem;
    /**
     * @var ReportStatusInterfaceFactory
     */
    private $reportStatusFactory;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * Processor constructor.
     * @param CollectionFactory $reportCollectionFactory
     * @param MailHelper $mailHelper
     * @param Csv $csvProcessor
     * @param DirectoryList $directoryList
     * @param Filesystem $filesystem
     * @param ReportStatusInterfaceFactory $reportStatusFactory
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        CollectionFactory $reportCollectionFactory,
        MailHelper $mailHelper,
        Csv $csvProcessor,
        DirectoryList $directoryList,
        Filesystem $filesystem,
        ReportStatusInterfaceFactory $reportStatusFactory,
        ResourceConnection $resourceConnection
    ) {
        $this->reportCollectionFactory = $reportCollectionFactory;
        $this->mailHelper = $mailHelper;
        $this->csvProcessor = $csvProcessor;
        $this->directoryList = $directoryList;
        $this->filesystem = $filesystem;
        $this->reportStatusFactory = $reportStatusFactory;
        $this->resourceConnection = $resourceConnection;
    }

    public function canProcess(ReportInterface $report): bool
    {
        // TODO: Implement canProcess() method.
    }

    /**
     * @param ReportInterface $report
     * @return ReportStatusInterface
     * @throws MailException
     */
    public function process(ReportInterface $report): ReportStatusInterface
    {
        $status = $this->reportStatusFactory->create();

        $reportOutput = $this->getConnection()->fetchAll($report->getSql(), [], Zend_Db::FETCH_ASSOC);

        // PRENDI IL DRIVER

        $csv = $this->produceReport($report, $reportOutput);

        //  Invia

        $this->sendReport($status);

        $count = count($reportOutput);

        $to = explode(',', $report->getReceiverEmails());
        $this->mailHelper->sendReportMail($to, [$csv], ['report_name' => $report->getName(), 'report_count' => $count]);

        return $status;
    }

    /**
     * @param ReportInterface $report
     * @param array $data
     * @return string
     * @throws FileSystemException
     */
    public function produceReport(ReportInterface $report, $data = [])
    {
        $fileDirectoryPath = $this->directoryList->
            getPath(DirectoryList::VAR_DIR) .
            DIRECTORY_SEPARATOR . 'stockafix' . DIRECTORY_SEPARATOR;

        if (!is_dir($fileDirectoryPath)) {
            if (!mkdir($fileDirectoryPath, 0777, true) && !is_dir($fileDirectoryPath)) {
                throw new RuntimeException(sprintf('Directory "%s" was not created', $fileDirectoryPath));
            }
        }

        $now = new DateTime();
        $fileName = $report->getName() . '_' . $now->format("YmdHis") . '.csv';
        $filePath = $fileDirectoryPath . $fileName;

        $this->csvProcessor
            ->setEnclosure('"')
            ->setDelimiter(';')
            ->appendData($filePath, $data);

        return $filePath;
    }

    /**
     * @param ReportInterface $report
     */
    public function sendReport(ReportStatusInterface $reportStatus)
    {
       $receivers = $reportStatus->getReport()->getReceivers();

       foreach ($receivers as $receiver)
       {
           $receiver->send($reportStatus);
       }
    }

    /**
     * @return AdapterInterface
     */
    public function getConnection()
    {
        if ($this->connection === null) {
            $this->connection = $this->resourceConnection->getConnection();
        }
        return $this->connection;
    }
}
